@@ -55,6 +55,7 @@ int main(int argc, char *argv[])
 	struct perftest_parameters	user_param;
 	struct perftest_comm		user_comm;
 	struct bw_report_data		my_bw_rep, rem_bw_rep;
+	int rdma_cm_flow_destroyed = 0;
 
 	/* init default values to user's parameters */
 	memset(&ctx, 0, sizeof(struct pingpong_context));
@@ -69,10 +70,10 @@ int main(int argc, char *argv[])
 	if (ret_parser) {
 		if (ret_parser != VERSION_EXIT && ret_parser != HELP_EXIT)
 			fprintf(stderr, " Parser function exited with Error\n");
-		return FAILURE;
+		goto return_error;
 	}
 
-	if (user_param.use_xrc && user_param.duplex) {
+	if ((user_param.connection_type == DC || user_param.use_xrc) && user_param.duplex) {
 		user_param.num_of_qps *= 2;
 	}
 
@@ -84,31 +85,27 @@ int main(int argc, char *argv[])
 	ctx.context = ibv_open_device(ib_dev);
 	if (!ctx.context) {
 		fprintf(stderr, " Couldn't get context for the device\n");
-		return FAILURE;
+		goto free_devname;
 	}
 
-	#ifdef HAVE_MASKED_ATOMICS
-	if (check_masked_atomics_support(&ctx)) {
-		user_param.masked_atomics = 1;
-		user_param.use_exp = 1;
+	/* Verify user parameters that require the device context,
+	 * the function will print the relevent error info. */
+	if (verify_params_with_device_context(ctx.context, &user_param))
+	{
+		fprintf(stderr, " Couldn't get context for the device\n");
+		goto free_devname;
 	}
 
-	if (user_param.masked_atomics && (user_param.work_rdma_cm || user_param.use_rdma_cm)) {
-		fprintf(stderr, "atomic test is not supported with -R/-z flag (rdma_cm) with this device.\n");
-		return FAILURE;
-	}
-	#endif
-
-	/* See if MTU and link type are valid and supported. */
+	/* See if link type is valid and supported. */
 	if (check_link(ctx.context, &user_param)) {
 		fprintf(stderr, " Couldn't get context for the device\n");
-		return FAILURE;
+		goto free_devname;
 	}
 
 	/* copy the relevant user parameters to the comm struct + creating rdma_cm resources. */
 	if (create_comm_struct(&user_comm, &user_param)) {
 		fprintf(stderr, " Unable to create RDMA_CM resources\n");
-		return FAILURE;
+		goto free_devname;
 	}
 
 	if (user_param.output == FULL_VERBOSITY && user_param.machine == SERVER) {
@@ -120,26 +117,31 @@ int main(int argc, char *argv[])
 	/* Initialize the connection and print the local data. */
 	if (establish_connection(&user_comm)) {
 		fprintf(stderr, " Unable to init the socket connection\n");
-		return FAILURE;
+		dealloc_comm_struct(&user_comm, &user_param);
+		goto free_devname;
 	}
 
 	exchange_versions(&user_comm, &user_param);
-
+	check_version_compatibility(&user_param);
 	check_sys_data(&user_comm, &user_param);
 
-	/* See if MTU and link type are valid and supported. */
+	/* See if MTU is valid and supported. */
 	if (check_mtu(ctx.context, &user_param, &user_comm)) {
 		fprintf(stderr, " Couldn't get context for the device\n");
-		return FAILURE;
+		dealloc_comm_struct(&user_comm, &user_param);
+		goto free_devname;
 	}
 
-	ALLOCATE(my_dest, struct pingpong_dest, user_param.num_of_qps);
+	MAIN_ALLOC(my_dest, struct pingpong_dest, user_param.num_of_qps, free_rdma_params);
 	memset(my_dest, 0, sizeof(struct pingpong_dest)*user_param.num_of_qps);
-	ALLOCATE(rem_dest, struct pingpong_dest, user_param.num_of_qps);
+	MAIN_ALLOC(rem_dest, struct pingpong_dest, user_param.num_of_qps, free_rem_dest);
 	memset(rem_dest, 0, sizeof(struct pingpong_dest)*user_param.num_of_qps);
 
 	/* Allocating arrays needed for the test. */
-	alloc_ctx(&ctx, &user_param);
+	if(alloc_ctx(&ctx,&user_param)){
+		fprintf(stderr, "Couldn't allocate context\n");
+		goto free_mem;
+	}
 
 	/* Create RDMA CM resources and connect through CM. */
 	if (user_param.work_rdma_cm == ON) {
@@ -148,59 +150,81 @@ int main(int argc, char *argv[])
 		if (rc) {
 			fprintf(stderr,
 				"Failed to create RDMA CM connection with resources.\n");
-			return FAILURE;
+			dealloc_ctx(&ctx, &user_param);
+			goto free_mem;
 		}
 	} else {
 		/* create all the basic IB resources. */
 		if (ctx_init(&ctx, &user_param)) {
 			fprintf(stderr, " Couldn't create IB resources\n");
-			return FAILURE;
+			dealloc_ctx(&ctx, &user_param);
+			goto free_mem;
 		}
 	}
 
 	/* Set up the Connection. */
 	if (set_up_connection(&ctx, &user_param, my_dest)) {
 		fprintf(stderr, " Unable to set up socket connection\n");
-		return FAILURE;
+		goto destroy_context;
 	}
 
 	/* Print basic test information. */
 	ctx_print_test_info(&user_param);
 
-	/* Print this machine QP information */
-	for (i = 0; i < user_param.num_of_qps; i++)
-		ctx_print_pingpong_data(&my_dest[i], &user_comm);
-
-	user_comm.rdma_params->side = REMOTE;
 	for (i = 0; i < user_param.num_of_qps; i++) {
 
 		/* shaking hands and gather the other side info. */
 		if (ctx_hand_shake(&user_comm, &my_dest[i], &rem_dest[i])) {
 			fprintf(stderr, "Failed to exchange data between server and clients\n");
-			return FAILURE;
+			goto destroy_context;
 		}
-		ctx_print_pingpong_data(&rem_dest[i], &user_comm);
 	}
 
 	if (user_param.work_rdma_cm == OFF) {
 		if (ctx_check_gid_compatibility(&my_dest[0], &rem_dest[0])) {
 			fprintf(stderr, "\n Found Incompatibility issue with GID types.\n");
 			fprintf(stderr, " Please Try to use a different IP version.\n\n");
-			return FAILURE;
+			goto destroy_context;
 		}
 	}
 
 	if (user_param.work_rdma_cm == OFF) {
 		if (ctx_connect(&ctx, rem_dest, &user_param, my_dest)) {
 			fprintf(stderr, " Unable to Connect the HCA's through the link\n");
-			return FAILURE;
+			goto destroy_context;
 		}
 	}
 
+	if (user_param.connection_type == DC)
+	{
+		/* Set up connection one more time to send qpn properly for DC */
+		if (set_up_connection(&ctx,&user_param,my_dest)) {
+			fprintf(stderr," Unable to set up socket connection\n");
+			goto destroy_context;
+		}
+	}
+
+	/* Print this machine QP information */
+	for (i=0; i < user_param.num_of_qps; i++) {
+		ctx_print_pingpong_data(&my_dest[i],&user_comm);
+	}
+
+	user_comm.rdma_params->side = REMOTE;
+
+	for (i=0; i < user_param.num_of_qps; i++) {
+
+		if (ctx_hand_shake(&user_comm,&my_dest[i],&rem_dest[i])) {
+			fprintf(stderr," Failed to exchange data between server and clients\n");
+			goto destroy_context;
+		}
+
+		ctx_print_pingpong_data(&rem_dest[i],&user_comm);
+	}
+
 	/* An additional handshake is required after moving qp to RTR. */
 	if (ctx_hand_shake(&user_comm, &my_dest[0], &rem_dest[0])) {
 		fprintf(stderr, "Failed to exchange data between server and clients\n");
-		return FAILURE;
+		goto destroy_context;
 	}
 
 	/* For half duplex tests, server just waits for client to exit */
@@ -213,7 +237,7 @@ int main(int argc, char *argv[])
 
 		if (ctx_hand_shake(&user_comm, &my_dest[0], &rem_dest[0])) {
 			fprintf(stderr, " Failed to exchange data between server and clients\n");
-			return FAILURE;
+			goto destroy_context;
 		}
 
 		xchg_bw_reports(&user_comm, &my_bw_rep, &rem_bw_rep, atof(user_param.rem_version));
@@ -225,26 +249,42 @@ int main(int argc, char *argv[])
 
 		if (ctx_close_connection(&user_comm, &my_dest[0], &rem_dest[0])) {
 			fprintf(stderr, "Failed to close connection between server and client\n");
-			return FAILURE;
+			goto destroy_context;
 		}
 
+
 		if (user_param.work_rdma_cm == ON) {
 			if (destroy_ctx(&ctx, &user_param)) {
 				fprintf(stderr, "Failed to destroy resources.\n");
+				goto destroy_cm_context;
+			}
+			free(my_dest);
+			free(rem_dest);
+			free(user_param.ib_devname);
+			user_comm.rdma_params->work_rdma_cm = OFF;
+			if(destroy_ctx(user_comm.rdma_ctx, user_comm.rdma_params)) {
+				free(user_comm.rdma_params);
+				free(user_comm.rdma_ctx);
 				return FAILURE;
 			}
-
-			user_comm.rdma_params->work_rdma_cm = OFF;
-			return destroy_ctx(user_comm.rdma_ctx, user_comm.rdma_params);
+			free(user_comm.rdma_params);
+			return SUCCESS;
 		}
-
-		return destroy_ctx(&ctx, &user_param);
+		free(my_dest);
+		free(rem_dest);
+		free(user_param.ib_devname);
+		if(destroy_ctx(&ctx, &user_param)) {
+			free(user_comm.rdma_params);
+			return FAILURE;
+		}
+		free(user_comm.rdma_params);
+		return SUCCESS;
 	}
 
 	if (user_param.use_event) {
 		if (ibv_req_notify_cq(ctx.send_cq, 0)) {
 			fprintf(stderr, "Couldn't request CQ notification\n");
-			return FAILURE;
+			goto free_mem;
 		}
 	}
 	if (user_param.output == FULL_VERBOSITY) {
@@ -260,20 +300,20 @@ int main(int argc, char *argv[])
 		if (user_param.perform_warm_up) {
 			if (perform_warm_up(&ctx, &user_param)) {
 				fprintf(stderr, "Problems with warm up\n");
-				return FAILURE;
+				goto free_mem;
 			}
 		}
 
 		if (user_param.duplex) {
 			if (ctx_hand_shake(&user_comm, &my_dest[0], &rem_dest[0])) {
 				fprintf(stderr, "Failed to sync between server and client between different msg sizes\n");
-				return FAILURE;
+				goto free_mem;
 			}
 		}
 
 		if (run_iter_bw(&ctx, &user_param)) {
 			fprintf(stderr, " Error occurred in run_iter function\n");
-			return FAILURE;
+			goto free_mem;
 		}
 
 		print_report_bw(&user_param, &my_bw_rep);
@@ -302,7 +342,7 @@ int main(int argc, char *argv[])
 
 		if (run_iter_bw_infinitely(&ctx, &user_param)) {
 			fprintf(stderr, " Error occurred while running infinitely! aborting ...\n");
-			return FAILURE;
+			goto free_mem;
 		}
 	}
 
@@ -314,7 +354,7 @@ int main(int argc, char *argv[])
 
 		if (ctx_hand_shake(&user_comm, &my_dest[0], &rem_dest[0])) {
 			fprintf(stderr, " Failed to exchange data between server and clients\n");
-			return FAILURE;
+			goto free_mem;
 		}
 
 		xchg_bw_reports(&user_comm, &my_bw_rep, &rem_bw_rep, atof(user_param.rem_version));
@@ -322,28 +362,75 @@ int main(int argc, char *argv[])
 
 	if (ctx_close_connection(&user_comm, &my_dest[0], &rem_dest[0])) {
 		fprintf(stderr, "Failed to close connection between server and client\n");
-		return FAILURE;
+		goto free_mem;
 	}
 
 	if (!user_param.is_bw_limit_passed && (user_param.is_limit_bw == ON)) {
 		fprintf(stderr, "Error: BW result is below bw limit\n");
-		return FAILURE;
+		goto destroy_context;
 	}
 
 	if (!user_param.is_msgrate_limit_passed && (user_param.is_limit_bw == ON)) {
 		fprintf(stderr, "Error: Msg rate  is below msg_rate limit\n");
-		return FAILURE;
+		goto destroy_context;
 	}
 
 	if (user_param.work_rdma_cm == ON) {
 		if (destroy_ctx(&ctx, &user_param)) {
 			fprintf(stderr, "Failed to destroy resources.\n");
-			return FAILURE;
+			goto destroy_cm_context;
 		}
 
 		user_comm.rdma_params->work_rdma_cm = OFF;
-		return destroy_ctx(user_comm.rdma_ctx, user_comm.rdma_params);
+		free(my_dest);
+		free(rem_dest);
+		free(user_param.ib_devname);
+		if(destroy_ctx(user_comm.rdma_ctx, user_comm.rdma_params)) {
+			free(user_comm.rdma_params);
+			free(user_comm.rdma_ctx);
+			return FAILURE;
+		}
+		free(user_comm.rdma_ctx);
+		free(user_comm.rdma_params);
+		return SUCCESS;
 	}
 
-	return destroy_ctx(&ctx, &user_param);
+	free(my_dest);
+	free(rem_dest);
+	free(user_param.ib_devname);
+
+	if(destroy_ctx(&ctx, &user_param)){
+		free(user_comm.rdma_params);
+		return FAILURE;
+	}
+	free(user_comm.rdma_params);
+	return SUCCESS;
+
+destroy_context:
+	if (destroy_ctx(&ctx,&user_param))
+		fprintf(stderr, "Failed to destroy resources\n");
+destroy_cm_context:
+	if (user_param.work_rdma_cm == ON) {
+		rdma_cm_flow_destroyed = 1;
+		user_comm.rdma_params->work_rdma_cm = OFF;
+		destroy_ctx(user_comm.rdma_ctx,user_comm.rdma_params);
+	}
+free_mem:
+	free(rem_dest);
+free_rem_dest:
+	free(my_dest);
+free_rdma_params:
+	if (user_param.use_rdma_cm == ON && rdma_cm_flow_destroyed == 0)
+		dealloc_comm_struct(&user_comm, &user_param);
+	else {
+		if (user_param.use_rdma_cm == ON)
+			free(user_comm.rdma_ctx);
+		free(user_comm.rdma_params);
+	}
+free_devname:
+	free(user_param.ib_devname);
+return_error:
+	//coverity[leaked_storage]
+	return FAILURE;
+
 }
