@@ -14,6 +14,8 @@ struct perftest_parameters;
 
 bool neuron_memory_supported();
 
+bool neuron_memory_dmabuf_supported();
+
 struct memory_ctx *neuron_memory_create(struct perftest_parameters *params);
 
 
@@ -23,6 +25,10 @@ inline bool neuron_memory_supported() {
 	return false;
 }
 
+inline bool neuron_memory_dmabuf_supported() {
+	return false;
+}
+
 inline struct memory_ctx *neuron_memory_create(struct perftest_parameters *params) {
 	return NULL;
 }
