@@ -102,17 +102,13 @@ static void mac_from_user(uint8_t   *mac, uint8_t *gid,int size )
 /******************************************************************************
  *
  ******************************************************************************/
-static uint16_t ip_checksum	(void * buf,size_t 	  hdr_len)
+static uint16_t ip_checksum(union IP_V4_header_raw *iph, size_t hdr_len)
 {
-	unsigned long sum = 0;
-	const uint16_t *ip1;
-	ip1 = buf;
-	while (hdr_len > 1) {
-		sum += *ip1++;
-		if (sum & 0x80000000)
-			sum = (sum & 0xFFFF) + (sum >> 16);
-		hdr_len -= 2;
-	}
+	size_t idx = hdr_len / 4;
+	unsigned long long sum = 0;
+
+	while (idx)
+		sum += iph->raw[--idx];
 	while (sum >> 16)
 		sum = (sum & 0xFFFF) + (sum >> 16);
 
@@ -150,23 +146,23 @@ void gen_ipv6_header(void* ip_header_buffer, uint8_t* saddr, uint8_t* daddr,
 void gen_ipv4_header(void* ip_header_buffer, uint32_t* saddr, uint32_t* daddr,
 		     uint8_t protocol, int pkt_size, int ttl, int tos, int flows_offset)
 {
-	struct IP_V4_header ip_header;
+	union IP_V4_header_raw raw;
 
-	memset(&ip_header,0,sizeof(struct IP_V4_header));
+	memset(&raw.ip_header, 0, sizeof(struct IP_V4_header));
 
-	ip_header.version = 4;
-	ip_header.ihl = 5;
-	ip_header.tos = (tos == DEF_TOS)? 0 : tos;
-	ip_header.tot_len = htons(pkt_size);
-	ip_header.id = htons(0);
-	ip_header.frag_off = htons(0);
-	ip_header.ttl = ttl;
-	ip_header.protocol = protocol;
-	ip_header.saddr = *saddr;
-	ip_header.daddr = *daddr;
-	ip_header.check = ip_checksum((void*)&ip_header,sizeof(struct IP_V4_header));
+	raw.ip_header.version = 4;
+	raw.ip_header.ihl = 5;
+	raw.ip_header.tos = (tos == DEF_TOS)? 0 : tos;
+	raw.ip_header.tot_len = htons(pkt_size);
+	raw.ip_header.id = htons(0);
+	raw.ip_header.frag_off = htons(0);
+	raw.ip_header.ttl = ttl;
+	raw.ip_header.protocol = protocol;
+	raw.ip_header.saddr = *saddr;
+	raw.ip_header.daddr = *daddr;
+	raw.ip_header.check = ip_checksum(&raw, sizeof(struct IP_V4_header));
 
-	memcpy(ip_header_buffer, &ip_header, sizeof(struct IP_V4_header));
+	memcpy(ip_header_buffer, &raw.ip_header, sizeof(struct IP_V4_header));
 }
 
 /******************************************************************************
@@ -206,13 +202,13 @@ void gen_tcp_header(void* TCP_header_buffer,int src_port ,int dst_port)
 /*****************************************************************************
  * generates a new ethernet header
  *****************************************************************************/
-void gen_eth_header(struct ETH_header* eth_header,uint8_t* src_mac,
-		uint8_t* dst_mac, uint16_t eth_type)
+void gen_eth_header(struct ETH_header* eth_header,uint8_t* src_mac, uint8_t* dst_mac, uint16_t eth_type,
+		    struct perftest_parameters* user_param, struct memory_ctx *memory)
 {
-	memcpy(eth_header->src_mac, src_mac, 6);
-	memcpy(eth_header->dst_mac, dst_mac, 6);
-	eth_header->eth_type = htons(eth_type);
-
+	uint16_t eth_type_temp = htons(eth_type);
+	memory->copy_host_to_buffer(eth_header->src_mac, src_mac, 6);
+	memory->copy_host_to_buffer(eth_header->dst_mac, dst_mac, 6);
+	memory->copy_host_to_buffer(&(eth_header->eth_type), &eth_type_temp, sizeof(uint16_t));
 }
 
 static int get_ip_size(int is_ipv6)
@@ -316,79 +312,103 @@ static char *etype_str(uint16_t etype)
 /******************************************************************************
  *
  ******************************************************************************/
-void print_ethernet_header(void* in_ethernet_header)
+void print_ethernet_header(void* in_ethernet_header, struct perftest_parameters* user_param, struct memory_ctx* memory)
 {
 	struct ETH_header* p_ethernet_header = in_ethernet_header;
+	uint8_t* p_src_mac;
+	uint8_t* p_dst_mac;
+	uint16_t p_eth_type;
 	if (NULL == p_ethernet_header) {
 		fprintf(stderr, "ETH_header pointer is Null\n");
 		return;
 	}
 
+	ALLOCATE(p_src_mac, uint8_t, 6);
+	ALLOCATE(p_dst_mac, uint8_t, 6);
+	memory->copy_buffer_to_host(p_src_mac, p_ethernet_header->src_mac, 6);
+	memory->copy_buffer_to_host(p_dst_mac, p_ethernet_header->dst_mac, 6);
+	memory->copy_buffer_to_host(&p_eth_type, &p_ethernet_header->eth_type, sizeof(uint16_t));
+
 	printf("**raw ethernet header****************************************\n\n");
 	printf("--------------------------------------------------------------\n");
 	printf("| Dest MAC         | Src MAC          | Packet Type          |\n");
 	printf("|------------------------------------------------------------|\n");
 	printf("|");
 	printf(PERF_MAC_FMT,
-			p_ethernet_header->dst_mac[0],
-			p_ethernet_header->dst_mac[1],
-			p_ethernet_header->dst_mac[2],
-			p_ethernet_header->dst_mac[3],
-			p_ethernet_header->dst_mac[4],
-			p_ethernet_header->dst_mac[5]);
+			p_dst_mac[0],
+			p_dst_mac[1],
+			p_dst_mac[2],
+			p_dst_mac[3],
+			p_dst_mac[4],
+			p_dst_mac[5]);
 	printf("|");
 	printf(PERF_MAC_FMT,
-			p_ethernet_header->src_mac[0],
-			p_ethernet_header->src_mac[1],
-			p_ethernet_header->src_mac[2],
-			p_ethernet_header->src_mac[3],
-			p_ethernet_header->src_mac[4],
-			p_ethernet_header->src_mac[5]);
+			p_src_mac[0],
+			p_src_mac[1],
+			p_src_mac[2],
+			p_src_mac[3],
+			p_src_mac[4],
+			p_src_mac[5]);
 	printf("|");
-	char* eth_type = etype_str((ntohs(p_ethernet_header->eth_type)));
+	char* eth_type = etype_str((ntohs(p_eth_type)));
 	printf("%-22s|\n",eth_type);
 	printf("|------------------------------------------------------------|\n\n");
 
+	free(p_src_mac);
+	free(p_dst_mac);
 }
 /******************************************************************************
 *
 ******************************************************************************/
-void print_ethernet_vlan_header(void* in_ethernet_header)
+void print_ethernet_vlan_header(void* in_ethernet_header, struct perftest_parameters* user_param, struct memory_ctx* memory)
 {
 	struct ETH_vlan_header* p_ethernet_header = in_ethernet_header;
-        if (NULL == p_ethernet_header) {
-                fprintf(stderr, "ETH_header pointer is Null\n");
-                return;
-        }
+	uint8_t* p_src_mac;
+	uint8_t* p_dst_mac;
+	uint16_t p_eth_type;
+	uint32_t p_vlan_header;
+	if (NULL == p_ethernet_header) {
+			fprintf(stderr, "ETH_header pointer is Null\n");
+			return;
+	}
 
-        printf("**raw ethernet header****************************************\n\n");
-        printf("----------------------------------------------------------------------------\n");
-        printf("| Dest MAC         | Src MAC          |    vlan tag    |   Packet Type     |\n");
-        printf("|--------------------------------------------------------------------------|\n");
-        printf("|");
-        printf(PERF_MAC_FMT,
-                        p_ethernet_header->dst_mac[0],
-                        p_ethernet_header->dst_mac[1],
-                        p_ethernet_header->dst_mac[2],
-                        p_ethernet_header->dst_mac[3],
-                        p_ethernet_header->dst_mac[4],
-                        p_ethernet_header->dst_mac[5]);
-        printf("|");
-        printf(PERF_MAC_FMT,
-                        p_ethernet_header->src_mac[0],
-                        p_ethernet_header->src_mac[1],
-                        p_ethernet_header->src_mac[2],
-                        p_ethernet_header->src_mac[3],
-                        p_ethernet_header->src_mac[4],
-                        p_ethernet_header->src_mac[5]);
-        printf("|");
-        printf("   0x%08x   ",ntohl(p_ethernet_header->vlan_header));
+	ALLOCATE(p_src_mac, uint8_t, 6);
+	ALLOCATE(p_dst_mac, uint8_t, 6);
+	memory->copy_buffer_to_host(p_src_mac, p_ethernet_header->src_mac, 6);
+	memory->copy_buffer_to_host(p_dst_mac, p_ethernet_header->dst_mac, 6);
+	memory->copy_buffer_to_host(&p_eth_type, &p_ethernet_header->eth_type, sizeof(uint16_t));
+	memory->copy_buffer_to_host(&p_vlan_header, &p_ethernet_header->vlan_header, sizeof(uint32_t));
 
-        printf("|");
-        char* eth_type = (ntohs(p_ethernet_header->eth_type) ==  IP_ETHER_TYPE ? "IP" : "DEFAULT");
-        printf("%-19s|\n",eth_type);
-        printf("|--------------------------------------------------------------------------|\n\n");
+	printf("**raw ethernet header****************************************\n\n");
+	printf("----------------------------------------------------------------------------\n");
+	printf("| Dest MAC         | Src MAC          |    vlan tag    |   Packet Type     |\n");
+	printf("|--------------------------------------------------------------------------|\n");
+	printf("|");
+	printf(PERF_MAC_FMT,
+					p_dst_mac[0],
+					p_dst_mac[1],
+					p_dst_mac[2],
+					p_dst_mac[3],
+					p_dst_mac[4],
+					p_dst_mac[5]);
+	printf("|");
+	printf(PERF_MAC_FMT,
+					p_src_mac[0],
+					p_src_mac[1],
+					p_src_mac[2],
+					p_src_mac[3],
+					p_src_mac[4],
+					p_src_mac[5]);
+	printf("|");
+	printf("   0x%08x   ",ntohl(p_vlan_header));
 
+	printf("|");
+	char* eth_type = (ntohs(p_eth_type) ==  IP_ETHER_TYPE ? "IP" : "DEFAULT");
+	printf("%-19s|\n",eth_type);
+	printf("|--------------------------------------------------------------------------|\n\n");
+
+	free(p_src_mac);
+	free(p_dst_mac);
 }
 /******************************************************************************
  *
@@ -494,7 +514,7 @@ void print_tcp_header(struct TCP_header* tcp_header)
  *
  ******************************************************************************/
 
-void print_pkt(void* pkt,struct perftest_parameters *user_param)
+void print_pkt(void* pkt,struct perftest_parameters *user_param, struct memory_ctx *memory)
 {
 
 	if (user_param->output != FULL_VERBOSITY)
@@ -505,7 +525,7 @@ void print_pkt(void* pkt,struct perftest_parameters *user_param)
 		return;
 	}
 
-	user_param->print_eth_func((struct ETH_header*)pkt);
+	user_param->print_eth_func((struct ETH_header*)pkt, user_param, memory);
 	if(user_param->is_client_ip || user_param->is_server_ip) {
 		// cppcheck-suppress arithOperationsOnVoidPointer
 		pkt = (void*)pkt + sizeof(struct ETH_header);
@@ -537,6 +557,7 @@ void build_pkt_on_buffer(struct ETH_header* eth_header,
 			 struct raw_ethernet_info *my_dest_info,
 			 struct raw_ethernet_info *rem_dest_info,
 			 struct perftest_parameters *user_param,
+			 struct memory_ctx *memory,
 			 uint16_t eth_type, uint16_t ip_next_protocol,
 			 int print_flag, int pkt_size, int flows_offset)
 {
@@ -552,14 +573,17 @@ void build_pkt_on_buffer(struct ETH_header* eth_header,
 		vlan_pcp = user_param->vlan_pcp;
 	}
 
-	gen_eth_header(eth_header, my_dest_info->mac, rem_dest_info->mac, eth_type);
+	gen_eth_header(eth_header, my_dest_info->mac, rem_dest_info->mac, eth_type, user_param, memory);
 
 	if(user_param->vlan_en) {
 		struct ETH_vlan_header *p_eth_vlan = (struct ETH_vlan_header *)eth_header;
-		p_eth_vlan->eth_type = eth_header->eth_type;
-		// cppcheck-suppress integerOverflow
-		p_eth_vlan->vlan_header = htonl(VLAN_TPID << 16 |
-								((vlan_pcp & 0x7) << 13) | VLAN_VID | VLAN_CFI << 12);
+
+		uint32_t vlan_header = htonl((uint32_t)VLAN_TPID << 16 |
+								((vlan_pcp & 0x7) << 13) | VLAN_VID | VLAN_CFI << 12);;
+
+		memory->copy_buffer_to_buffer(&p_eth_vlan->eth_type, &eth_header->eth_type, sizeof(uint16_t));
+		memory->copy_host_to_buffer(&p_eth_vlan->vlan_header, &vlan_header, sizeof(uint32_t));
+
 		eth_header_size = sizeof(struct ETH_vlan_header);
 		pkt_size -=4;
 	}
@@ -597,7 +621,7 @@ void build_pkt_on_buffer(struct ETH_header* eth_header,
 	}
 
 	if(print_flag == PRINT_ON) {
-		print_pkt((void*)eth_header, user_param);
+		print_pkt((void*)eth_header, user_param, memory);
 	}
 }
 
@@ -640,7 +664,7 @@ void create_raw_eth_pkt( struct perftest_parameters *user_param,
 			/* fill ctx buffer with same packets */
 			while ((flow_limit - INC(ctx->size, ctx->cache_line_size)) >= pkt_offset) {
 				build_pkt_on_buffer(eth_header, my_dest_info, rem_dest_info,
-						    user_param, eth_type, ip_next_protocol,
+						    user_param, ctx->memory, eth_type, ip_next_protocol,
 						    print_flag, ctx->size - RAWETH_ADDITION, i);
 				print_flag = PRINT_OFF;
 				pkt_offset += INC(ctx->size, ctx->cache_line_size);/* update the offset to next packet in same flow */
@@ -655,7 +679,7 @@ void create_raw_eth_pkt( struct perftest_parameters *user_param,
 			// cppcheck-suppress arithOperationsOnVoidPointer
 			eth_header = (void*)buf + pkt_offset;
 			build_pkt_on_buffer(eth_header, my_dest_info, rem_dest_info,
-					    user_param, eth_type, ip_next_protocol,
+					    user_param, ctx->memory, eth_type, ip_next_protocol,
 					    PRINT_ON, ctx->size - RAWETH_ADDITION, i);
 
 		}
@@ -708,9 +732,10 @@ static void fill_ip_common(struct ibv_flow_spec* spec_info,
 			}
 			if (user_param->flow_label) {
 				ipv6_spec->val.flow_label =
-					htonl(user_param->flow_label);
+					htonl(user_param->flow_label[user_param->flow_label[1] % user_param->flow_label[0] + 2]);
 				ipv6_spec->mask.flow_label =
 					htonl(0xfffff);
+				user_param->flow_label[1] = user_param->flow_label[1] + 1;
 			}
 			memcpy((void*)&ipv6_spec->mask.dst_ip, ipv6_mask, 16);
 			memcpy((void*)&ipv6_spec->mask.src_ip, ipv6_mask, 16);
@@ -810,12 +835,13 @@ int set_up_flow_rules(
 			spec_info->tcp_udp.val.dst_port = htons(local_port);
 			spec_info->tcp_udp.val.src_port = htons(remote_port);
 		} else {
+			//coverity[overrun-local]
 			spec_info->tcp_udp.val.src_port = htons(local_port);
 			spec_info->tcp_udp.val.dst_port = htons(remote_port);
 		}
 
-		memset((void*)&spec_info->tcp_udp.mask.dst_port, 0xFF,sizeof(spec_info->ipv4.mask.dst_ip));
-		memset((void*)&spec_info->tcp_udp.mask.src_port, 0xFF,sizeof(spec_info->ipv4.mask.src_ip));
+		memset((void*)&spec_info->tcp_udp.mask.dst_port, 0xFF, sizeof(spec_info->tcp_udp.mask.dst_port));
+		memset((void*)&spec_info->tcp_udp.mask.src_port, 0xFF, sizeof(spec_info->tcp_udp.mask.src_port));
 	}
 
 	if (user_param->is_ethertype) {
@@ -1021,8 +1047,7 @@ int run_iter_fw(struct pingpong_context *ctx,struct perftest_parameters *user_pa
 		}
 
 		if (user_param->use_event) {
-
-			if (ctx_notify_events(ctx->channel)) {
+			if (ctx_notify_send_recv_events(ctx)) {
 				fprintf(stderr, "Failed to notify events to CQ");
 				return_value = FAILURE;
 				goto cleaning;
